# %{version}, %{rel} are provided by the Makefile
# interactive build by defining, ie:
# rpmbuild -bb --define="rel 1" --define="version 3.1.90" lsb-test-python.spec
%define pversion4 2.4.2
%define pversion5 2.5
%define docdir /opt/lsb/doc/%{name}
%define sname python-test-%{version}
Summary: LSB Python Test Suite
Name: lsb-test-python
Version: %{version}
Release: %{rel}
License: GPL-compatible 
Group: Development/Tools
Source: %{sname}.tar.gz
Source1: http://www.python.org/ftp/python/%{pversion4}/Python-%{pversion4}.tar.bz2
Source2: http://www.python.org/ftp/python/%{pversion5}/Python-%{pversion5}.tar.bz2
Source3: tetj.py
Patch0: python-system-python.patch
Patch1: python-tet-test.patch
Patch2: python-test-new-behavior.patch
Patch3: python-test-private-testcapi.patch
Patch4: python-test-handle-26.patch
URL: http://www.linuxbase.org/test
#Prefix: %{_prefix}
BuildRoot: %{_tmppath}/%{name}-root
AutoReqProv: no
BuildRequires: lsb-appchk-python

%description
This is the official package version of the LSB Python test suite. 
These tests are derived from Python's build-time tests.

#==================================================
%prep
%setup -q -c -b1 -b2
%patch1 -p0 -b .tet-output
pushd Python-%{pversion4}/Lib/test
%patch0 -p0 -b .system-python
popd
%patch2 -p0 -b .new-behavior
%patch3 -p0 -b .testcapi
pushd Python-%{pversion5}/Lib/test
%patch4 -p0 -b .26
popd

#==================================================
%build
# thanks to bug 2282 we now need to build _testcapi.so 
# how to do that without building the whole mess ?
# unicode option to avoid undefined symbol PyUnicodeUCS2_Decode 
for pyver in %{pversion4} %{pversion5};do
  cd Python-$pyver
  CC=lsbcc CXX=lsbc++ ./configure --enable-unicode=ucs4
  make
  cd ..
done

#==================================================
%install

rm -rf ${RPM_BUILD_ROOT}
mkdir -p ${RPM_BUILD_ROOT}/opt/lsb/test/python
mkdir -p ${RPM_BUILD_ROOT}/opt/lsb/test/python/Lib{24,25}/test
mkdir -p ${RPM_BUILD_ROOT}/opt/lsb/test/python/results
mkdir -p ${RPM_BUILD_ROOT}/opt/lsb/test/python/lib{24,25}

# _testcapi.so module
for pyver in %{pversion4} %{pversion5};do
  if [ "$pyver" = %{pversion4} ];then
    mlib=lib24
  else
    mlib=lib25
  fi
  find Python-$pyver/build -name _testcapi.so | xargs -i cp {} ${RPM_BUILD_ROOT}/opt/lsb/test/python/$mlib 
done

# new - be selective about what we copy
for pyver in %{pversion4} %{pversion5};do
  if [ "$pyver" = %{pversion4} ];then
    tlib=Lib24
  else
    tlib=Lib25
  fi

  # the test harness and support code
  for test in __init__ regrtest;do 
    cp Python-$pyver/Lib/test/$test.py ${RPM_BUILD_ROOT}/opt/lsb/test/python/$tlib/test/
  done
  for test in support normalization multibytecodec_support importhooks;do
    cp Python-$pyver/Lib/test/test_$test.py ${RPM_BUILD_ROOT}/opt/lsb/test/python/$tlib/test/
  done

  # builtins - not sure about all of these
  for test in augassign binop bool bufio builtin call \
	class compare compile complex contains decorators \
	enumerate eof file format frozen generators genexps \
	getargs getargs2 global grammar hash hexoct \
	import isinstance iter iterlen largefile list \
	long longexp module multibytecodec mutants \
	opcodes operations pow scope set sets slice sort \
	str syntax trace tuple types ucn unary unicode \
	univnewlines unpack xrange;do
    cp Python-$pyver/Lib/test/test_$test.py ${RPM_BUILD_ROOT}/opt/lsb/test/python/$tlib/test/
  done

  # only install the tests for modules defined in the spec
  for mod in `grep -v '^#' /opt/lsb/share/appchk/lsb-python-modules.list`;do 
    if [ -f Python-$pyver/Lib/test/test_$mod.py ];then
      cp Python-$pyver/Lib/test/test_$mod.py ${RPM_BUILD_ROOT}/opt/lsb/test/python/$tlib/test/
    fi
  done

  # names do not map for these
  # _{cn,hk,jp,kr,tw} -> test_codecencodings
  # these are builtins too
  for test in _cn _hk _jp _kr _tw;do
    cp Python-$pyver/Lib/test/test_codecencodings$test.py ${RPM_BUILD_ROOT}/opt/lsb/test/python/$tlib/test/
  done

  # support files
  cp Python-$pyver/Lib/test/cjkencodings_test.py ${RPM_BUILD_ROOT}/opt/lsb/test/python/$tlib/test/

  # names do not map for these either
  # cPickle -> cpickle, cStringIO -> StringIO
  # deque is part of collections
  # _bisect -> bisect, _heapq -> heapq, _hotshot -> hotshot
  # normalization is part of unicodedata 
  # poll is part of select 
  # richcmp is part of operator
  # threaded_import, threadsignals  is part of thread
  # unicode_file is for unicodedata
  for test in bisect cpickle deque heapq hotshot normalization \
	poll richcmp threaded_import threadsignals \
	unicode_file StringIO;do
    cp Python-$pyver/Lib/test/test_$test.py ${RPM_BUILD_ROOT}/opt/lsb/test/python/$tlib/test/
  done
  # goes with test_threaded_import on 2.5
  if [ "$tlib" = "Lib25" ]; then
    cp Python-$pyver/Lib/test/threaded_import_hangers.py ${RPM_BUILD_ROOT}/opt/lsb/test/python/$tlib/test/
    # need both 25 and 26 versions of these two
    cp Python-$pyver/Lib/test/test_{generators,genexps}.py.26 ${RPM_BUILD_ROOT}/opt/lsb/test/python/$tlib/test/
  fi

  # support files
  for test in double_const list_tests pickletester\
	seq_tests string_tests re_tests mapping_tests;do
    cp Python-$pyver/Lib/test/$test.py ${RPM_BUILD_ROOT}/opt/lsb/test/python/$tlib/test/
  done

  # not run - but are checked for
  for test in socket_ssl timeout;do
    cp Python-$pyver/Lib/test/test_$test.py ${RPM_BUILD_ROOT}/opt/lsb/test/python/$tlib/test/
  done
  cp Python-$pyver/Lib/test/test_codecmaps_* ${RPM_BUILD_ROOT}/opt/lsb/test/python/$tlib/test/

  # and the additional support files
  for sfile in txt uue tar xml out html db au 1;do
    cp Python-$pyver/Lib/test/*.$sfile ${RPM_BUILD_ROOT}/opt/lsb/test/python/$tlib/test/
  done 

  # datafiles and subdirs
  cp -pr Python-$pyver/Lib/test/decimaltestdata ${RPM_BUILD_ROOT}/opt/lsb/test/python/$tlib/test/
  cp -pr Python-$pyver/Lib/test/output ${RPM_BUILD_ROOT}/opt/lsb/test/python/$tlib/test/

done

cp -p %{sname}/scripts/runtime/run_tests ${RPM_BUILD_ROOT}/opt/lsb/test/python
chmod +x ${RPM_BUILD_ROOT}/opt/lsb/test/python/run_tests
cp -p %{sname}/scripts/runtime/single_test ${RPM_BUILD_ROOT}/opt/lsb/test/python
chmod +x ${RPM_BUILD_ROOT}/opt/lsb/test/python/single_test

# our tet functions
mkdir -p ${RPM_BUILD_ROOT}/opt/lsb/test/python/tet
cp -p %{SOURCE3} ${RPM_BUILD_ROOT}/opt/lsb/test/python/tet

# skip and known failures lists
mkdir -p ${RPM_BUILD_ROOT}/opt/lsb/test/python/lists
cp -p %{sname}/lists/skip* ${RPM_BUILD_ROOT}/opt/lsb/test/python/lists
cp -p %{sname}/lists/known* ${RPM_BUILD_ROOT}/opt/lsb/test/python/lists

# license and README.lsb-test
cat > README.lsb-test << EOF
Tests derived from the Python %{pversion4},%{pversion5} build-time test suite
Packaged by the Linux Foundation for LSB compliance testing
EOF

cat > README.single-test << EOF
To run a single test:
cd /opt/lsb/test/python
PYTHONPATH=\`pwd\`/Lib{24,25}:\`pwd\`/tet /usr/bin/python ./Lib{24,25}/test/regrtest.py some_test
(some_test would be something like test_global)
To get more verbose output, use regrtest.py -v
Or use the "single_test" script.
EOF

# VERSION file for the journal
cat > VERSION << EOF
%{name} %{version}-%{rel} (noarch)
EOF
cp VERSION ${RPM_BUILD_ROOT}/opt/lsb/test/python

# old console %%post messages to README
cat > README.%{name} << EOF
To run the tests, please execute the following command:
    cd /opt/lsb/test/python; ./run_tests
 
You will need to run the tests as root, or chown all of the files
to a user you want to run the tests as.
EOF

# minimal docs  
mkdir -p ${RPM_BUILD_ROOT}%{docdir}
cp Python-%{pversion4}/LICENSE LICENSE-%{pversion4}
cp Python-%{pversion5}/LICENSE LICENSE-%{pversion5}
cp README.lsb-test README.single-test README.%{name} LICENSE-%{pversion4} LICENSE-%{pversion5} ${RPM_BUILD_ROOT}%{docdir}

# clean up any stray .pyc files
pushd ${RPM_BUILD_ROOT}/opt/lsb/test/python
find . -name '*.pyc' | xargs rm -f

# clean up the patch backup files
find . -name '*.system-python' | xargs rm -f
find . -name '*.tet-output' | xargs rm -f
find . -name '*.new-tests' | xargs rm -f
popd

# deal with v2.6
cd ${RPM_BUILD_ROOT}/opt/lsb/test/python
  install -d Lib26/test
  cd Lib26/test
    ln -s ../../Lib25/test/* .
  cd ..
  ln -s lib25 lib26
cd ${RPM_BUILD_ROOT}/opt/lsb/test/python/lists
ln -s skip_test_25.txt skip_test_26.txt

# had trouble patching these two to work on both 2.5 and 2.6, use seperate tests
cd ${RPM_BUILD_ROOT}/opt/lsb/test/python
for test in generators genexps; do
  cd Lib26/test
    rm -f test_$test.py*
    cp -f ../../Lib25/test/test_$test.py .
  cd ../../Lib25/test
    mv test_$test.py.26 test_$test.py
  cd ../..
done

#==================================================
%clean
if [ ! -z "${RPM_BUILD_ROOT}"  -a "${RPM_BUILD_ROOT}" != "/" ]; then 
    rm -rf ${RPM_BUILD_ROOT}
fi

#======================================================================
%post

#==================================================
%files
%defattr(-,root,root)

%dir /opt/lsb/test/python
/opt/lsb/test/python/*
%dir %{docdir}
%{docdir}/*

#==================================================
%changelog
* Fri Oct 31 2008 Stew Benedict <stewb@linux-foundation.org>
- finish dealing with Python-2.6 (bug 2363)

* Mon Oct 20 2008 Stew Benedict <stewb@linux-foundation.org>
- symlink Lib26 -> Lib25, lib26->lib25 so tests at least try 
  to run against 2.6
 
* Thu Sep 11 2008 Stew Benedict <stewb@linux-foundation.org>
- build/package/use private _testcapi.so (bug 2282, P3)

* Fri Feb 22 2008 Stew Benedict <stewb@linux-foundation.org>
- move %%post console messages to README.%{name} (bug 1940)

* Tue Dec 18 2007 Stew Benedict <stewb@linux-foundation.org>
- back off subtests (P2) that have changed in 2.5.1

* Thu Dec 13 2007 Stew Benedict <stewb@linux-foundation.org>
- specifically use /usr/bin/python (Bug 1836)
- use tetj.py, fix journal output (Bug 1833, rework P1, add S3)

* Tue Jul 17 2007 Stew Benedict <stewb@linux-foundation.org>
- reflect changes to master module list
- remove pyexpat, strop, symtable, linuxaudiodev
- use grep -v '^#' on the blessed module list instead of cat
- codecs_*, multibytecodecs are builtin, using "_" - keep
- 108 tests now

* Wed Jul 11 2007 Stew Benedict <stewb@linux-foundation.org>
- add a build note comment at the top
- rebuild against new lsbappchk-python to pickup the changes
  _foo -> foo, and added os, re, string
- tweak the spec file a bit for addition of re
- add mapping_tests for test_os, test_weakref
- 111 tests now
- skipped tests now are treated as FAIL (P1)

* Fri Jul 06 2007 Stew Benedict <stewb@linux-foundation.org>
- try to limit the test list to what LSB specifies
- fix README.single-test
- 105 tests for both 2.4 and 2.5 now

* Tue Jun 05 2007 Stew Benedict <stewb@linux-foundation.org>
- initial packaging

